/*###############################################################################
# Linux Management Providers (LMP), OS provider package
# Copyright (C) 2007 Ilsoo Byun, ETRI <widepis@etri.re.kr ,widepis@empal.com>
# 
# This program is being developed under the "OpenDRIM" project.
# The "OpenDRIM" project web page: http://opendrim.sourceforge.net
# The "OpenDRIM" project mailing list: opendrim@googlegroups.com
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2
# of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#################################################################################

#################################################################################
# To contributors, please leave your contact information in this section
# AND comment your changes in the source code.
# 
# Modified by 2009 Khahramon NURIDDINOV, TUIT <qahramon0786@gmail.com>
# Modified by 2009 Guillaume BOTTEX, ETRI <guillaumebottex@etri.re.kr, guillaumebottex@gmail.com>
###############################################################################*/

#include "assocOpenDRIM_OperatingSystemStatistics.h"

int OS_OpenDRIM_OperatingSystemStatistics_associatorFilter(const CMPIBroker* broker, const CMPIContext* ctx, const CMPIObjectPath* cop, Instance& knownInstance, const char* assocClass, const char* resultClass, const char* role, const char* resultRole, bool& leftToRight, string& errorMessage) {
	_E_;
	string _assocClass, _resultClass, _resultRole, _role;
	assocClass == NULL ? _assocClass = "null" : _assocClass = assocClass;
	resultClass == NULL ? _resultClass = "null" : _resultClass = resultClass;
	resultRole == NULL ?  _resultRole = "null" :  _resultRole = resultRole;
	role == NULL ?  _role = "null" :  _role = role;
	
	for (unsigned int i=0; i<sizeof(OpenDRIM_OperatingSystemStatistics_classnames)/sizeof(char*); i++) {
		if (CF_strCmpNoCase(_assocClass, OpenDRIM_OperatingSystemStatistics_classnames[i])) {
			_assocClass=OpenDRIM_OperatingSystemStatistics_classnames[0];
			break;
		}
	}
	if (!CF_strCmpNoCase(_assocClass, "null") && !CF_strCmpNoCase(_assocClass, OpenDRIM_OperatingSystemStatistics_classnames[0]))
		return OK;
	
	Objectpath known(broker, (CMPIObjectPath*) cop);
	int errorCode = CA_associatorResolveDirection(broker, Element_classnames, Stats_classnames, Element_NAMESPACE, Stats_NAMESPACE, Element_role, Stats_role, known, _resultClass, _role, _resultRole, leftToRight);
	if (errorCode != OK)
		return -1;
	
	if (leftToRight) {
		CF_assert(OS_OpenDRIM_OperatingSystemStatistics_getElement(broker, ctx, known, knownInstance, NULL, errorMessage));
	} else {
		CF_assert(OS_OpenDRIM_OperatingSystemStatistics_getStats(broker, ctx, known, knownInstance, NULL, errorMessage));
	}
	_L_;
	return OK;
}

int OS_OpenDRIM_OperatingSystemStatistics_associator(const CMPIBroker* broker, const CMPIContext* ctx, const CMPIObjectPath* cop, Instance& knownInstance, const char* assocClass, const char* resultClass, const char* role, const char* resultRole, const char** properties, vector<Instance>& associatedInstances, bool& leftToRight, string& errorMessage, const string& dicriminant) {
	_E_;
	int errorCode = OS_OpenDRIM_OperatingSystemStatistics_associatorFilter(broker, ctx, cop, knownInstance, assocClass, resultClass, role, resultRole, leftToRight, errorMessage);
	if (errorCode == -1)
		return OK;
	if (errorCode != OK)
		return errorCode;
	
	if (leftToRight) {
		if (role != NULL && strcasecmp(role, "Element") != 0) return OK;
		if (resultRole != NULL && strcasecmp(resultRole, "Stats") != 0) return OK;
		CF_assert(OS_OpenDRIM_OperatingSystemStatistics_associatorElementToStats(broker, ctx, knownInstance, properties, associatedInstances, errorMessage, dicriminant));
	} else {
		if (role != NULL && strcasecmp(role, "Stats") != 0) return OK;
		if (resultRole != NULL && strcasecmp(resultRole, "Element") != 0) return OK;
		CF_assert(OS_OpenDRIM_OperatingSystemStatistics_associatorStatsToElement(broker, ctx, knownInstance, properties, associatedInstances, errorMessage, dicriminant));
	}
	_L_;
	return OK;
}

int OS_OpenDRIM_OperatingSystemStatistics_enumerateElements(const CMPIBroker* broker, const CMPIContext* ctx, vector<Instance>& instances, const char** properties, bool onlyNames, string& errorMessage) {
	_E_;
	/*
	 * CMPI broker version
	 */
	if (onlyNames) {
		vector<Objectpath> objectpaths;
		CF_assert(CB_enumerateInstanceNames(broker, ctx, Element_NAMESPACE, Element_classnames[0], objectpaths, errorMessage));
		for (size_t i=0; i<objectpaths.size(); i++)
			instances.push_back(Instance(broker, objectpaths[i]));
	} else {
		CF_assert(CB_enumerateInstances(broker,ctx, Element_NAMESPACE, Element_classnames[0], properties, instances, errorMessage));
	}
	/*
	 * Direct Invocation version
	 */
	/*
	vector<OpenDRIM_OperatingSystem> _instances;
	if (onlyNames) {
		CF_assert(OS_OpenDRIM_OperatingSystem_retrieve(broker, ctx, _instances, properties, errorMessage, "ein"));
	} else {
		CF_assert(OS_OpenDRIM_OperatingSystem_retrieve(broker, ctx, _instances, properties, errorMessage, "ei"));
	}

	for (size_t i=0; i<_instances.size(); i++)
		instances.push_back(Instance(broker, OS_OpenDRIM_OperatingSystem_toCMPIInstance(broker, _instances[i])));
	*/
	_L_;
	return OK;
}

int OS_OpenDRIM_OperatingSystemStatistics_getElement(const CMPIBroker* broker, const CMPIContext* ctx, const Objectpath& objectpath, Instance& instance, const char** properties, string& errorMessage) {
	_E_;
	if (!CF_strCmpNoCase(objectpath.getClassname(), Element_classnames[0]))
		return INVALID_CLASS;
	//if (!CF_strCmpNoCase(objectpath.getNamespace(), Element_NAMESPACE))
	//	return INVALID_NAMESPACE;
	/*
	 * CMPI broker version
	 */
	CF_assert(CB_getInstance(broker, ctx, objectpath, properties, instance, errorMessage));
	
	/*
	 * Direct Invocation version
	 */
	/*
	OpenDRIM_OperatingSystem _instance;
	OS_OpenDRIM_OperatingSystem_toCPP(broker, objectpath.getHdl(), _instance);
	CF_assert(OS_OpenDRIM_OperatingSystem_getInstance(broker, ctx, _instance, properties, errorMessage));
	instance = Instance(broker, OS_OpenDRIM_OperatingSystem_toCMPIInstance(broker, _instance));
	*/
	_L_;
	return OK;
}

int OS_OpenDRIM_OperatingSystemStatistics_enumerateStatss(const CMPIBroker* broker, const CMPIContext* ctx, vector<Instance>& instances, const char** properties, bool onlyNames, string& errorMessage) {
	_E_;
	/*
	 * CMPI broker version
	 */
	if (onlyNames) {
		vector<Objectpath> objectpaths;
		CF_assert(CB_enumerateInstanceNames(broker, ctx, Stats_NAMESPACE, Stats_classnames[0], objectpaths, errorMessage));
		for (size_t i=0; i<objectpaths.size(); i++)
			instances.push_back(Instance(broker, objectpaths[i]));
	} else {
		CF_assert(CB_enumerateInstances(broker,ctx, Stats_NAMESPACE, Stats_classnames[0], properties, instances, errorMessage));
	}
	/*
	 * Direct Invocation version
	 */
	/*
	vector<OpenDRIM_OperatingSystemLoadAverages> _instances;
	if (onlyNames) {
		CF_assert(OS_OpenDRIM_OperatingSystemLoadAverages_retrieve(broker, ctx, _instances, properties, errorMessage, "ein"));
	} else {
		CF_assert(OS_OpenDRIM_OperatingSystemLoadAverages_retrieve(broker, ctx, _instances, properties, errorMessage, "ei"));
	}

	for (size_t i=0; i<_instances.size(); i++)
		instances.push_back(Instance(broker, OS_OpenDRIM_OperatingSystemLoadAverages_toCMPIInstance(broker, _instances[i])));
	*/
	_L_;
	return OK;
}

int OS_OpenDRIM_OperatingSystemStatistics_getStats(const CMPIBroker* broker, const CMPIContext* ctx, const Objectpath& objectpath, Instance& instance, const char** properties, string& errorMessage) {
	_E_;
	if (!CF_strCmpNoCase(objectpath.getClassname(), Stats_classnames[0]))
		return INVALID_CLASS;
	//if (!CF_strCmpNoCase(objectpath.getNamespace(), Stats_NAMESPACE))
	//	return INVALID_NAMESPACE;
	/*
	 * CMPI broker version
	 */
	CF_assert(CB_getInstance(broker, ctx, objectpath, properties, instance, errorMessage));

	/*
	 * Direct Invocation version
	 */
	/*
	OpenDRIM_OperatingSystemLoadAverages _instance;
	OS_OpenDRIM_OperatingSystemLoadAverages_toCPP(broker, objectpath.getHdl(), _instance);
	CF_assert(OS_OpenDRIM_OperatingSystemLoadAverages_getInstance(broker, ctx, _instance, properties, errorMessage));	
	instance = Instance(broker, OS_OpenDRIM_OperatingSystemLoadAverages_toCMPIInstance(broker, _instance));
	*/
	_L_;
	return OK;
}
